<?php if(!defined('_VALID')){header('Status: 404 Not Found');die;}
## FS class
# @author legolas558
#
# centralized filesystem operations

include_once $GLOBALS['d_root'].'lang/'.$GLOBALS['my']->lang.'/admin/classes/fs.php';

define('_FS_WRITE_MODE', octdec($GLOBALS['d_writemode']));

class FS {

var $_batch_mode;
var $_critical_mode;

	function FS($failback = false, $batch = false) {
		// atomical batch sequences not yet supported
		$this->_batch_mode = $batch;
		// choose to fail (with server error) or to simply report back a false return value
		$this->_critical_mode = !$failback;
	}

	// opens a file for writeability and returns FALSE on failure
	function write_open($filename) {
		$f = @fopen($filename, 'wb');
		if ($f===false) {
			if ($this->_critical_mode) {
				$msg = _FS_WRITE_ACCESS_DENIED;
				global $my;
				if ($my->is_admin())
					$msg.=sprintf(' %s', '<small>'.fix_root_path($filename).'</small>');
				$this->_failure($msg);
			}
			return false;
		}
		return $f;
	}

	// opens a file in append mode and returns FALSE on failure
	function append_open($filename) {
		$f = @fopen($filename, 'a');
		if ($f===false) {
			if ($this->_critical_mode) {
				$msg = _FS_WRITE_ACCESS_DENIED;
				global $my;
				if ($my->is_admin())
					$msg.=sprintf(' %s', '<small>'.fix_root_path($filename).'</small>');
				$this->_failure($msg);
			}
			return false;
		}
		return $f;
	}

	function put_contents($filename, $content) {
		if (@file_put_contents($filename, $content)===false) {
			if ($this->_critical_mode) {
				$msg = _FS_WRITE_FAILURE;
				global $my;
				if ($my->is_admin())
					$msg.=sprintf(' %s', '<small>'.fix_root_path($filename).'</small>');
				$this->_failure($msg);
			}
			return false;
		}
		return true;
	}
	
	function _failure($msg) {
		global $d,$d_root, $my;
		include $d_root.'includes/servererror.php';
//		ob_end_clean();
//		ob_start();	// ???
		$explanation = '<hr />';
		if ($my->is_admin()) {
			$explanation.=_FS_FAILURE_EXPLANATION;
			$d->log(5, 'Filesystem: '.strip_tags($msg));
		} else {
			$explanation.=_FS_CONTACT_ADMIN;
			$d->log(5, 'Filesystem: '.$msg);
		}
		service_msg(_FS_OPERATION_FAILED, $msg, $explanation, 'stop');
		exit;
	}
	
	// assert the existance of a path (with trailing slash)
	// if $external is false, $path is relative to $d_root
	function assertpath($path, $ofs = 0, $external = false) {
//		echo 'Asserting '.substr($path, 0, $ofs).'<br>';
		$p = strpos($path, '/', $ofs);
		if (!$external) $r = $GLOBALS['d_root']; else $r='';
		if ($p===false) {
			if ($ofs===0)
				return $this->assertdir($r.$path);
			else
				return $this->assertdir($r.substr($path,0,$ofs));
		}
		return ($this->assertdir($r.substr($path, 0, $p)) && $this->assertpath($path, $p+1, $external));
	}
	
	function dir_exists($path) {
		return (/*file_exists($path) || */ is_dir($path));
	}

	// assert that $path is a directory, eventually creating it
	function assertdir($path, $mode = _FS_WRITE_MODE) {
		if (!$this->dir_exists($path))
			return $this->mkdir($path, $mode);
		return true;
	}
	
	function mkdir($path, $mode = _FS_WRITE_MODE) {
		if (!@mkdir($path, $mode)) {
			if ($this->_critical_mode) {
				$msg = _FS_MKDIR_FAILURE;
				global $my;
				if ($my->is_admin())
					$msg.=sprintf(' %s', '<small>'.fix_root_path($path).'</small>');
				$this->_failure($msg);
			}
			return false;
		}
		return true;
	}
	
	// warning: $source must exist before calling!
	function copy($source, $dest) {
		if (!@copy($source, $dest)) {
			// A bug of PHP on Windows...
			if (@filesize($source)===0)
				return true;
			if ($this->_critical_mode) {
				$msg = _FS_COPY_FAILURE;
				global $my;
				if ($my->is_admin())
					$msg.=' '.sprintf(_FS_SRC_TO_DST, '<small>'.fix_root_path($source).'</small>', '<small>'.fix_root_path($dest).'</small>');
				$this->_failure($msg);
			}
			return false;
		}
		return true;
	}
	
	function _raw_move($source, $dest) {
		// first try a copy/delete approach
		if (!@copy($source, $dest)) {
			if ($this->_critical_mode) {
				$msg = _FS_MOVE_FAILURE;
				global $my;
				if ($my->is_admin())
					$msg.=' '.sprintf(_FS_SRC_TO_DST, '<small>'.fix_root_path($source).'</small>', '<small>'.fix_root_path($dest).'</small>');
				$this->_failure($msg);
			}
			return false;
		}
		return $this->remove($source);
	}
	
	function move($source, $dest) {
		if (!@rename($source, $dest))
			// first try a copy/delete approach
			return $this->_raw_move($source, $dest);
		return true;
	}
	
	function assert_remove($path) {
		if (!file_exists($path))
			return true;
		return $this->remove($path);
	}
	
	function remove($path) {
		if (!@unlink($path)) {
			if (is_windows()) {	// try to remove the readonly flag
				@chmod($path, $GLOBALS['d_writemode']);
				if (@unlink($path))
					return true;
			}
			if ($this->_critical_mode) {
				$msg = _FS_REMOVE_FAILURE;
				global $my;
				if ($my->is_admin())
					$msg.=sprintf(' %s', '<small>'.fix_root_path($path).'</small>');
				$this->_failure($msg);
			}
			return false;
		}
		return true;
	}
	
	function rmdir($path) {
		if (!@rmdir($path)) {
			// performa a last attempt after chmodding the directory
			if (@chmod($path, $GLOBALS['d_writemode']) && @rmdir($path))
				return true;
			if (!$this->_critical_mode)
				return false;
			$msg = _FS_RMDIR_FAILURE;
			global $my;
			if ($my->is_admin())
				$msg.=sprintf(' %s', '<small>'.fix_root_path($path).'</small>');
			
			if (count(read_dir($path)))
				$msg.=' '._FS_FILES_WITHIN_FOLDER;
			$this->_failure($msg);
			return false;
		}
		return true;
	}

	// copies all files and folders from $src to $to (recursively)
	function xcopy($src, $to, $assert_paths = true) {
		$done = $this->fcopy($src, $to);
		$folders = read_dir($src, 'dir');
		foreach ($folders as $dir) {
			if ($assert_paths)
				$this->assertdir($to.$dir);
			$done += $this->xcopy($src.$dir.'/', $to.$dir.'/', $assert_paths);
		}
		return $done;
	}

	// as ::xcopy(), but moves the files
	function xmove($src, $to, $assert_paths = true) {
		$done = $this->fmove($src, $to);
		$folders = read_dir($src, 'dir');
		foreach ($folders as $dir) {
			if ($assert_paths)
				$this->assertdir($to.$dir);
			$done += $this->xmove($src.$dir.'/', $to.$dir.'/', $assert_paths);
		}
		return $done;
	}

	
	// copies only folders and files in subfolders (does not copy normal files in the $src folder)
	function tcopy($src, $to) {
		$done = 0;
		$folders = read_dir($src, 'dir');
		foreach ($folders as $dir) {
			$this->assertdir($to.$dir);
			$done += $this->fcopy($src.$dir.'/', $to.$dir.'/');
		}
		return $done;
	}
	
	// moves only files from the specified directory
	function fmove($src, $to) {
		$done = 0;
		$files = read_dir($src, 'file');
		foreach ($files as $file) {
			if ($this->move($src.$file, $to.$file)!==FALSE)
				++$done;
		}
		return $done;
	}

	// copies only files from the specified directory
	function fcopy($src, $to) {
		$done = 0;
		$files = read_dir($src, 'file');
		foreach ($files as $file) {
			if ($this->copy($src.$file, $to.$file)!==FALSE)
				++$done;
		}
		return $done;
	}

	// recursively delete a directory
	// must use trailing slash
	function purge($path) {
		if (!is_dir($path)) {
			if ($this->_critical_mode) {
				$msg = _FS_DIR_NOT_FOUND;
				global $my;
				if ($my->is_admin())
					$msg.=sprintf(' %s', '<small>'.fix_root_path($path).'</small>');
					$this->_failure($msg);
				}
			return false;
		}
		$handle = opendir($path);
		$noerror = true;
		while (false !== ($file = readdir($handle))) {
			if ($file[0] == '.') continue;
			if (is_dir($path.$file)) {
				$this->purge ($path.$file.'/');
				if(!$this->rmdir($path.$file))
					$noerror = false;
			} else if(!$this->unlink($path.$file))
				$noerror = false;
		}
		closedir($handle);
		return $noerror;
	}

	// recursively delete a directory and finally remove it too
	// must use trailing slash
	function deldir($dir)
	{
		if ($this->purge($dir))
			return $this->rmdir($dir);
		else
			return false;
	}
	
	function chmod($filename, $mode = _FS_WRITE_MODE) {
		if (!@chmod($filename, $mode)) {
			if (!$this->_critical_mode)
				return false;
			$msg = _FS_CHMOD_FAILURE;
			global $my;
			if ($my->is_admin())
				$msg.=sprintf(' %s', '<small>'.fix_root_path($filename).'</small>');
			$this->_failure($msg);
			return false;
		}
		return true;
	}

	function chmod_all($path, $mode) {
		$handle = opendir($path);
		$noerror = $this->chmod($path, $mode);
		if (!$noerror) return false;
		while (false !== ($file = readdir($handle))) {
			if ($file[0] == '.') continue;
			if (is_dir($path.$file)) {
				$noerror = $this->chmod_all ($path.$file."/", $mode);
				if (!$noerror) break;
			} else {
				$noerror = $this->chmod($path.$file, $mode);
				if (!$noerror) break;
			}
		}
		closedir($handle);
		return $noerror;
	}

	
	// deprecated methods
	function is_dir($d) { return $this->dir_exists($d); }
	function rename($src, $dest) { return $this->move($src, $dest); }
	function unlink($file) { return $this->remove($file); }
	function delete($file) { return $this->remove($file); }

}

?>